from starlette.middleware.base import BaseHTTPMiddleware
from starlette.requests import Request
from .utils.jwt import verify_token, create_access_token
from starlette.responses import Response, JSONResponse, RedirectResponse
import random


def generate_secret():
    return "".join(
        [
            random.choice(
                "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
            )
            for i in range(32)
        ]
    )


class HenauAuthMiddleware(BaseHTTPMiddleware):
    def __init__(
        self,
        app,
        login_router: str = "/login",
        excluded_routes: list[str] = None,
        redirect_url: str = None,
        get_user_func: callable = None,
        oauth2_user_func: callable = None,
        expires_delta: int = 3600,
        jwt_secret: str = generate_secret(),
        enable_permission_controller=True,
        *args,
        **keywords,
    ) -> None:
        """_summary_

        Args:
            app (_type_): ASGI APP 应用对象
            login_router (str, optional): 登录路由. Defaults to "/login".
            excluded_routes (list[str], optional): 排除的路由. Defaults to None.
            redirect_url (str, optional): 重定向地址. Defaults to None , 鉴权失败时自动重定向，为空则不启用.
            get_user_func (callable, optional): 获取用户方法，不传将默认使用payload. Defaults to None.
            oauth2_user_func (callable, optional): 用户授权方法，接受code 返回用户信息载体. Defaults to None.
            expires_delta (int, optional): 令牌过期时间. Defaults to 3600.
            jwt_secret (str, optional): 令牌秘钥. Defaults to "".join( [ random.choice( "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789" ) for i in range(32) ] ).
            enable_permission_controller (bool, optional): 是否启用权限控制. Defaults to True 启用后 request中会携带一个casbin 权限控制对象.
        """
        super().__init__(app, *args, **keywords)
        self.login_router = login_router
        self.excluded_routes = excluded_routes if excluded_routes else []
        self.redirect_url = redirect_url
        self.get_user_func = get_user_func
        self.oauth2_user_func = oauth2_user_func
        self.jwt_secret = jwt_secret
        self.expires_delta = expires_delta
        self.enable_permission_controller = enable_permission_controller

    def auth(self, request: Request):
        if request.url.path == self.login_router:
            code = request.query_params.get("code")
            try:
                payload = self.oauth2_user_func(code)
            except Exception as e:
                return JSONResponse(status_code=401, content={"message": str(e)})
            request.state.payload = payload
            request.state.user = (
                self.get_user_func(payload) if self.get_user_func else payload
            )
            request.state.token = create_access_token(
                payload, expires_delta=self.expires_delta, jwt_secret=self.jwt_secret
            )
        else:
            if request.url.path not in self.excluded_routes:
                if request.headers.get("Authorization") is None:
                    if self.redirect_url:
                        return RedirectResponse(self.redirect_url)
                    return JSONResponse(
                        status_code=401, content={"message": "未提供令牌"}
                    )
                else:
                    token = request.headers.get("Authorization").split(" ")[1]
                    try:
                        payload = verify_token(token, jwt_secret=self.jwt_secret)
                    except Exception:
                        if self.redirect_url:
                            return RedirectResponse(self.redirect_url)
                        return JSONResponse(
                            status_code=401, content={"message": "令牌错误"}
                        )
                    request.state.payload = payload
                    request.state.user = (
                        self.get_user_func(payload) if self.get_user_func else payload
                    )

    def permission_controller(self, request: Request):
        if self.enable_permission_controller:
            from ..casbin import get_enforcer

            request.state.e = get_enforcer()

    async def dispatch(self, request: Request, call_next):
        funcs = [self.auth, self.permission_controller]
        for func in funcs:
            res = func(request)
            if isinstance(res, Response):
                return res

        return await call_next(request)



from .super_auth import api_get_access_token_and_openid, api_get_user_info
from .super_auth import AccessTokenAndOpenId, UserInfo

__all__ = [
    "api_get_access_token_and_openid",
    "api_get_user_info",
    "AccessTokenAndOpenId",
    "UserInfo",
]



from typing import Optional, Union, Dict, Iterator, TypedDict, List
from mcp.client.sse import sse_client
from mcp import ClientSession
from ollama import AsyncClient


class MCPServerStdio(TypedDict):
    command: str
    args: List[str]


class MCPServerSSE(TypedDict):
    host: str


class HenauAILLM:
    """
    河南农业大学人工智能平台大模型推理补全接口封装
    """

    def __init__(self, appid: str, secret: str):
        """
        初始化接口
        :param appid: 应用唯一标识
        :param secret: 应用密钥
        """
        self.appid = appid
        self.secret = secret

        self.tools = []
        self.mcp_sessions = {}

        self.ollama_client = AsyncClient(
            host="https://oauth.henau.edu.cn/oauth2_ai_server/llm_completions/appid/nd51f865dece11bdfe/secret/09f6dd2aef714c71d8e7145c8598b6c8/style/ollama/api/chat"
        )

    def completions_v1(
        self,
        model: str,
        messages: Optional[str] = None,
        stream: bool = False,
    ) -> Union[Dict, Iterator[Dict]]:
        return self.ollama_client.chat(
            model=model, messages=messages, tools=self.tools, stream=stream
        )

    def completions(
        self, model: str, messages: Optional[str] = None, stream: bool = False
    ) -> Union[Dict, Iterator[Dict]]:
        return self.completions_v1(model, messages, stream)

    async def add_mcp_server(
        self, server_name: str, mcp_server_config: MCPServerStdio | MCPServerSSE
    ):
        if server_name in self.mcp_sessions.keys():
            print(
                "\nServer already exists. if you sure it is a new server , please rename it"
            )
            return

        if "command" in mcp_server_config.keys() and "args" in mcp_server_config.keys():
            pass
        elif "host" in mcp_server_config.keys():
            # print("SSE mode")
            self._streams_context = sse_client(url=mcp_server_config["host"])
            streams = await self._streams_context.__aenter__()
            # print("get streams")
            self._session_context = ClientSession(*streams)
            # print("get session context")
            session: ClientSession = await self._session_context.__aenter__()
            # print("get session")
            await session.initialize()
            # print("init session")
            self.mcp_sessions[server_name] = session
            # print("add session")
            response = await session.list_tools()
            # print("get tools")
            self.tools.extend(
                [
                    {
                        "type": "function",
                        "function": {
                            "name": f"{server_name}___{tool.name}",
                            "description": tool.description,
                            "parameters": tool.inputSchema,
                        },
                    }
                    for tool in response.tools
                ]
            )



# Import necessary modules
from peewee import SqliteDatabase

# Import the casbin peewee adapter
import casbin_peewee_adapter

# Import casbin
import casbin

# Import the os module
import os

# Define the database
DATABAEE = SqliteDatabase("casbin.sqlite")
# Bind the database to the CasbinRule model
casbin_peewee_adapter.CasbinRule._meta.database = DATABAEE
# Create the CasbinRule table if it doesn't exist
DATABAEE.create_tables([casbin_peewee_adapter.CasbinRule])
# Create the adapter
adapter = casbin_peewee_adapter.Adapter(database=DATABAEE)


# Define the function to get the enforcer
def get_enforcer() -> casbin.Enforcer:
    # Get the path to the model configuration file
    model_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "model.conf")
    # Create the enforcer
    e = casbin.Enforcer(model_path, adapter, True)
    # Return the enforcer
    return e



from peewee import Query
from typing import Tuple, Any, Dict
from .to_dict import to_dict
from typing import TypeVar, Generic, List
from pydantic import BaseModel


def paginate(
    query: Query, page: int = 1, size: int = 10, **kewargs
) -> Tuple[list, Dict[str, Any]]:
    """
    Peewee 通用分页方法

    Args:
        query: peewee 查询对象
        page: 当前页码，从1开始
        size: 每页记录数

    Returns:
        tuple: (分页结果列表, 分页信息字典)
    """
    page = int(page)
    size = int(size)
    if page < 1:
        page = 1
    if size < 1:
        size = 10

    total = query.count()
    total_pages = (total + size - 1) // size

    if page > total_pages:
        page = total_pages

    results = query.paginate(page, size)

    pagination_info = {
        "page": page,
        "size": size,
        "total": total,
        "total_pages": total_pages,
        "has_prev": page > 1,
        "has_next": page < total_pages,
    }

    return {"records": to_dict(results, **kewargs), "pagination": pagination_info}


T = TypeVar("T")


class PaginationModel(BaseModel):
    page: int
    size: int
    total: int
    total_pages: int
    has_prev: bool
    has_next: bool


class PageSchemas(BaseModel, Generic[T]):
    records: List[T]
    pagination: PaginationModel



from .super_message import (
    api_send_message_by_weixin_super,
    api_send_message_by_sms_super,
)
from .super_message import Data
from .base_message import api_send_message_by_sms_base, api_send_message_by_weixin_base

__all__ = [
    "api_send_message_by_sms_base",
    "api_send_message_by_weixin_base",
    "api_send_message_by_sms_super",
    "api_send_message_by_weixin_super",
    "Data",
]



import jwt
from datetime import datetime, timedelta


# 验证令牌是否有效
def verify_token(token: str, jwt_secret: str = None, jwt_algorithm: str = "HS256"):
    """_summary_

    Args:
        token (str): 令牌
        jwt_secret (str, optional): JWT 秘钥. Defaults to None.
        jwt_algorithm (str, optional): JWT加密算法. Defaults to "HS256".

    Returns:
        dict: 解密后的数据
    """
    payload = jwt.decode(token, jwt_secret, algorithms=jwt_algorithm)
    return payload


# 生成 JWT 令牌的函数
def create_access_token(
    data: dict,
    expires_delta: int = 300,
    jwt_secret: str = None,
    jwt_algorithm: str = "HS256",
) -> str:
    """_summary_

    Args:
        data (dict): 数据载体
        expires_delta (int, optional): 过期时间单位分钟. Defaults to 300.
        jwt_secret (str, optional): JWT秘钥. Defaults to None.
        jwt_algorithm (str, optional): JWT加密算法. Defaults to "HS256".

    Returns:
        str : 加密后的令牌
    """
    to_encode = data.copy()

    if expires_delta:
        expire = datetime.utcnow() + timedelta(minutes=expires_delta)

    to_encode.update({"exp": expire})  # 在令牌中加入过期时间
    encoded_jwt = jwt.encode(to_encode, jwt_secret, algorithm=jwt_algorithm)  # 加密令牌
    return encoded_jwt



from playhouse.shortcuts import model_to_dict
from peewee import ModelSelect, Model


def to_dict(obj, **kewargs):
    if isinstance(obj, Model):
        return model_to_dict(obj, **kewargs)
    elif isinstance(obj, dict):
        return {k: to_dict(v, **kewargs) for k, v in obj.items()}
    elif isinstance(obj, list):
        return [to_dict(item, **kewargs) for item in obj]
    elif isinstance(obj, ModelSelect):
        return [to_dict(item, **kewargs) for item in obj]
    else:
        return obj



"多条件构造器"


def splicing_conditions(model, sql, query):
    for i in query.model_dump(exclude_none=True).keys():
        if isinstance(getattr(query, i), list):
            sql = sql.where(getattr(model, i) in (getattr(query, i)))
        if isinstance(getattr(query, i), str):
            sql = sql.where(getattr(model, i) % f"%{getattr(query, i)}%")
        else:
            sql = sql.where(getattr(model, i) == getattr(query, i))
    return sql


def splicing_conditions_dict(model, sql, query: dict, useAll=False):
    try:
        for i in query.keys():
            try:
                if not query[i]:
                    continue
                if useAll and query[i] == "all":
                    continue
                if query[i] in ["true", "false"]:
                    sql = sql.where(
                        getattr(model, i) == (1 if query[i] == "true" else 0)
                    )
                elif isinstance(query[i], list):
                    sql = sql.where(getattr(model, i) in query[i])
                elif isinstance(query[i], str):
                    if query[i] == "":
                        continue
                    sql = sql.where(getattr(model, i) % f"%{query[i]}%")
                else:
                    sql = sql.where(getattr(model, i) == query[i])
            except AttributeError:
                # 如果模型没有这个属性，则跳过
                pass
    except AttributeError:
        return sql
    return sql



from .oauth2 import api_get_access_token_and_openid, api_get_user_info
from .oauth2 import AccessTokenAndOpenId, UserInfo
from .message import (
    api_send_message_by_sms_base,
    api_send_message_by_weixin_base,
    api_send_message_by_weixin_super,
    api_send_message_by_sms_super,
)
from .message import Data

from starlette.middleware.base import BaseHTTPMiddleware
from .fastapi.oauth_henau import HenauAuthMiddleware
from functools import partial
from typing import Literal

policy_enum = Literal["super_auth", "auth", "permission"]


class HenauAuth:
    def __init__(
        self, app_id: str, app_secret: str, base_url: str = "https://oauth.henau.edu.cn"
    ):
        """_summary_

        Args:
            app_id (str): APP ID
            app_secret (str): APP Secret
            base_url (_type_, optional): oauth2 地址. Defaults to "https://oauth.henau.edu.cn".
        """
        self.app_id = app_id
        self.app_secret = app_secret
        self.base_url = base_url

    def get_access_token_and_open_id(self, code: str) -> AccessTokenAndOpenId:
        """_summary_

        Args:
            code (str): 前端获取的code

        Returns:
            dict[str , str]: 包含access_token和henau_openid的字典
        """
        data = api_get_access_token_and_openid(
            self.app_id, self.app_secret, code, host=self.base_url
        )
        return data

    def get_user_info_by_access_token(
        self, access_token: str, henau_openid: str
    ) -> UserInfo:
        """_summary_

        Args:
            access_token (str): 通行令牌
            henau_openid (str): 用户唯一标识

        Returns:
            dict[str , str]: 用户信息
        """
        access_token_and_open_id = api_get_user_info(
            access_token, henau_openid, host=self.base_url
        )
        return access_token_and_open_id

    def get_user_info_super(self, code: str) -> UserInfo:
        """_summary_

        Args:
            code (_type_): 前端获得的code

        Returns:
            UserInfo: 获得的用户信息
        """
        access_token_and_open_id = self.get_access_token_and_open_id(code)
        user_info = self.get_user_info_by_access_token(
            access_token_and_open_id["access_token"],
            access_token_and_open_id["henau_openid"],
        )
        return user_info

    def send_message_by_weixin(
        self, henau_openid: str, template_number: str, jump_link: str, data: Data
    ) -> bool:
        """_summary_

        Args:
            henau_openid (str): 用户唯一标识
            template_number (str): 消息模版编号
            jump_link (str): 跳转链接
            data (Data): 数据载体

        Returns:
            bool: 是否成功
        """
        res = api_send_message_by_weixin_base(
            app_id=self.app_id,
            app_secret=self.app_secret,
            henau_openid=henau_openid,
            data=data,
            template_number=template_number,
            jump_link=jump_link,
            host=self.base_url,
        )
        return res

    def send_message_by_sms(
        self, henau_openid: str, template_number: str, data: Data
    ) -> bool:
        """_summary_

        Args:
            henau_openid (str): 用户唯一标识
            template_number (str): 短信模版编号
            data (Data): 数据载体

        Returns:
            bool: 是否成功
        """

        res = api_send_message_by_sms_base(
            app_id=self.app_id,
            app_secret=self.app_secret,
            henau_openid=henau_openid,
            data=data,
            template_number=template_number,
            host=self.base_url,
        )
        return res

    def send_message_by_weixin_super(
        self,
        name: str,
        henau_number: str,
        template_number: str,
        jump_link: str,
        data: Data,
    ) -> bool:
        """_summary_

        Args:
            name (str): 发送目标用户的名字
            henau_number (str): 发送目标用户的学工号
            template_number (str): 发送信息的模版编号
            jump_link (str): 跳转链接
            data (Data): 参数

        Returns:
            bool: 是否发送成功
        """
        res = api_send_message_by_weixin_super(
            app_id=self.app_id,
            app_secret=self.app_secret,
            name=name,
            henau_number=henau_number,
            data=data,
            template_number=template_number,
            jump_link=jump_link,
            host=self.base_url,
        )
        return res

    def send_message_by_sms_super(
        self, name: str, henau_number: str, template_number: str, data: Data
    ) -> bool:
        """_summary_

        Args:
            name (str): 发送目标用户的名字
            henau_number (str): 发送目标用户的学工号
            template_number (str): 发送信息的模版编号
            data (Data): 参数

        Returns:
            bool : 是否发送成功
        """
        res = api_send_message_by_sms_super(
            app_id=self.app_id,
            app_secret=self.app_secret,
            name=name,
            henau_number=henau_number,
            data=data,
            template_number=template_number,
            host=self.base_url,
        )
        return res

    def get_fastapi_middleware(
        self, policy: policy_enum = "super_auth"
    ) -> BaseHTTPMiddleware:
        """_summary_

        Returns:
            FastAPI Middleware: 返回一个FastAPI中间件,自动完成鉴权，令牌分发，令牌校验
        """
        get_user_funcs = {"super_auth": self.get_user_info_super}
        return partial(HenauAuthMiddleware, oauth2_user_func=get_user_funcs[policy])

    def get_fastapi_admin_server_middleware(self) -> BaseHTTPMiddleware:
        """
        admin server 是一个公共管理后台，目前仅可以管理用户和日志，开发者仅仅需要简单配置此中间件
        就可以在admin server中管理该应用 有关admin_server 的功能大致如下
        - 用户管理, 用户查看，自定义用户状态
        - 日志管理
        - 数据仪表盘
        - 数据管理
        """
        pass



import os
from pathlib import Path
import typer


def create_file_tree(directory_structure, root_path="."):
    """
    根据多维字典创建文件树

    参数:
        directory_structure: 多维字典，键是文件/文件夹名，字符串值作为文件内容
        root_path: 文件树的根目录路径，默认为当前目录
    """
    root_path = Path(root_path)

    for name, content in directory_structure.items():
        current_path = root_path / name

        if isinstance(content, dict):
            # 如果是字典，创建文件夹并递归处理
            os.makedirs(current_path, exist_ok=True)
            create_file_tree(content, current_path)
        elif isinstance(content, str):
            # 如果是字符串，创建文件并写入内容
            with open(current_path, "w", encoding="utf-8") as f:
                f.write(content)
        else:
            # 其他类型，转换为字符串写入
            with open(current_path, "w", encoding="utf-8") as f:
                f.write(str(content))


app = typer.Typer()


@app.command()
def fastapi():
    """
    生成 FastAPI 项目文件
    """
    from .template.fastapi import fastapi_template

    create_file_tree(fastapi_template, root_path=".")


@app.command()
def list():
    """
    列出所有命令
    """
    print("fastapi: 生成 FastAPI 项目文件")


def main(*args, **kwargs):
    app(*args, **kwargs)



import requests
from typing import TypedDict


class AccessTokenAndOpenId(TypedDict):
    access_token: str
    henau_openid: str


def api_get_access_token_and_openid(
    app_id, app_secret, code, host: str = "https://oauth.henau.edu.cn"
) -> AccessTokenAndOpenId:
    url = (
        f"{host}"
        f"/oauth2_server"
        f"/access_token"
        f"?appid={app_id}"
        f"&secret={app_secret}"
        f"&code={code}"
        f"&grant_type=authorization_code"
    )

    response = requests.request("GET", url)

    if response.json()["status"] != "error":
        data = response.json().get("data")
        return {
            "access_token": data.get("access_token"),
            "henau_openid": data.get("henau_openid"),
        }
    else:
        raise Exception("code失效，获取access_token , openid失败")


class UserInfo(TypedDict):
    henau_openid: str
    user_name: str
    user_number: str
    user_section: str
    user_phone: str | None  # 如需该字段，请向信息化办公室单独申请
    user_nickname: str | None  # 如需该字段，请向信息化办公室单独申请
    user_avatar_url: str | None  # 如需该字段，请向信息化办公室单独申请
    user_status: int


def api_get_user_info(
    access_token: str, henau_openid: str, host: str = "https://oauth.henau.edu.cn"
) -> UserInfo:
    url = (
        f"{host}"
        f"/oauth2_server/userinfo"
        f"?access_token={access_token}"
        f"&henau_openid={henau_openid}"
    )

    response = requests.request("GET", url)

    if response.status_code == 200:
        data = response.json().get("data")
        return data
    else:
        raise Exception("获取信息失败")



fastapi_template = {
    "fastapi_project": {
        "README.md": "# FastAPI Project\n\nThis is a FastAPI project generated automatically.",
        ".gitignore": "__pycache__/\n*.py[cod]\n*$py.class\n.env\n.venv\nvenv/\nENV/\n.env.local\n.env.development\n.env.test\n.env.production\n*.log\n",
        "app": {
            "__init__.py": "# Initialize app package",
            "config.py": """
            import yaml
from pathlib import Path
from typing import Any, Dict, Optional


class Settings:
    def __init__(self, yaml_file: str = "config.yaml"):
        self._yaml_file = Path(yaml_file)
        self._load_settings()

    def _load_settings(self) -> None:
        if not self._yaml_file.exists():
            raise FileNotFoundError(f"配置文件 {self._yaml_file} 不存在")

        with open(self._yaml_file, "r", encoding="utf-8") as f:
            config_data = yaml.safe_load(f) or {}

        self._set_attributes(config_data)

    def _set_attributes(self, config_data: Dict[str, Any]) -> None:
        for key, value in config_data.items():
            if isinstance(value, dict):
                # 对于嵌套字典，创建一个嵌套的 Settings 对象
                setattr(self, key, Settings._from_dict(value))
            else:
                setattr(self, key, value)

    @classmethod
    def _from_dict(cls, data: Dict[str, Any]) -> "Settings":
        instance = cls.__new__(cls)
        instance._yaml_file = None  # 标记这不是从文件加载的实例
        instance._set_attributes(data)
        return instance

    def __repr__(self) -> str:
        attrs = []
        for key in sorted(self.__dict__.keys()):
            if not key.startswith("_"):
                value = getattr(self, key)
                attrs.append(f"{key}={repr(value)}")
        return f"Settings({', '.join(attrs)})"

    def reload(self) -> None:
        if self._yaml_file is None:
            raise RuntimeError("无法重新加载，此实例是从字典创建的")
        self._load_settings()


# 创建配置实例
settings = Settings()

            """,
            "main.py": """from fastapi import FastAPI, Request
from src.henau_auth import HenauAuth

app = FastAPI()

client = HenauAuth(
    app_id="", # 应用id
    app_secret="", # 应用密钥
    base_url="https://oauth.henau.edu.cn" # 授权服务器地址 默认为 https://oauth.henau.edu.cn
)


app.add_middleware(
    client.get_fastapi_middleware(),
    login_router="/login", # 登录接口的路由，默认为 /login
    excluded_routes=["/test"], # 排除的路由，被排除的路由不会进行鉴权，默认为 []
    jwt_secret="dasdsafdsfsregtrjukiuok", # jwt的密钥，不传值则每次动态生成一个 建议传值32位字符串即可
    expires_delta=3600, # jwt过期时间，默认为3600分钟，单位分钟
    # get_user_func = lambda payload: User.get_or_none(User.open_id == payload["henau_openid"]) 
    # 如果传入了get_user_func，则会在获取到用户信息后调用该函数，该函数可以返回一个用户对象，该用户对象会被存储在request.state.user中
)


@app.get("/login")
async def login(request: Request, code: str):
    return {"user": request.state.user, "token": request.state.token}


@app.get("/other")
async def user(request: Request):
    return {"user": request.state.user}


@app.get("/test")
async def test(request: Request, code: str = None):
    return {"code": code, "user": request.state}
""",
            "database.py": """from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

SQLALCHEMY_DATABASE_URL = "sqlite:///./sql_app.db"

engine = create_engine(
    SQLALCHEMY_DATABASE_URL, connect_args={"check_same_thread": False}
)
SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)

Base = declarative_base()""",
            "routers": {
                "__init__.py": "",
            },
        },
        "tests": {
            "__init__.py": "",
            "test_main.py": """from fastapi.testclient import TestClient
from app.main import app

client = TestClient(app)

def test_read_root():
    response = client.get("/")
    assert response.status_code == 200
    assert response.json() == {"message": "Hello World"}""",
        },
    }
}



from typing import List

from peewee import (
    Model,
    CharField,
    TextField,
    DateTimeField,
    DateField,
    TimeField,
    SQL,
    IntegerField,
    ColumnMetadata,
    ForeignKeyField,
)
from playhouse.pool import PooledMySQLDatabase
from playhouse.shortcuts import ReconnectMixin
from .page import paginate
from .construction import splicing_conditions_dict


class ReconnectMySQLDatabase(ReconnectMixin, PooledMySQLDatabase):
    pass


field_type = {
    "text": TextField,
    "datetime": DateTimeField,
    "timestamp": DateTimeField,
    "tinyint": IntegerField,
    "json": TextField,
    "date": DateField,
    "time": TimeField,
    "varchar": CharField,
    "int": IntegerField,
}


def data_to_attr(column: ColumnMetadata):
    return field_type[column.data_type](
        null=column.null, constraints=[SQL(f"DEFAULT {column.default}")]
    )


def generate_attr(columns: List[ColumnMetadata]):
    attr_dict = {}
    for column in columns:
        attr_dict[column.name] = data_to_attr(column)
    return attr_dict


class DbClient:
    def __init__(self, db: ReconnectMySQLDatabase):
        self.db = db
        self.BaseModel = type(
            "BaseModel",
            (Model,),
            {
                "Meta": type("Meta", (), dict(database=db)),
            },
        )
        self.models = {}

    def add_model(self):
        pass

    def add_all_models(self):
        for table in self.db.get_tables():
            self.models[table] = type(
                table,
                (self.BaseModel,),
                {
                    **generate_attr(self.db.get_columns(table)),
                    "Meta": type("Meta", (), dict(table_name=table)),
                },
            )
        for table in self.db.get_tables():
            for foreign_key in self.db.get_foreign_keys(table):
                setattr(
                    self.models[table],
                    foreign_key.dest_table,
                    ForeignKeyField(
                        column_name=foreign_key.column,
                        field=foreign_key.dest_column,
                        model=self.models[foreign_key.dest_table],
                    ),
                )

    def create_tables(self):
        self.db.create_tables(self.models.values())

    def insert(self):
        pass

    def page_query(self, table_name: str, query: dict = {}):
        model = self.models[table_name]
        sql = splicing_conditions_dict(model, model.select(), query)
        return paginate(sql, page=query.get("page") or 1, size=query.get("size") or 10)

    def create(self, table_name: str, body: dict = {}):
        model = Model(self.models[table_name])
        model.create(**body)

    def update(self, table_name: str, body: dict = {}):
        model = self.models[table_name]
        record = model.get_by_id(body.get("id"))
        for key, value in body.items():
            setattr(record, key, value)
        record.save()



from starlette.middleware.base import BaseHTTPMiddleware
from starlette.responses import JSONResponse
import requests


class AdminServer(BaseHTTPMiddleware):
    def __init__(
        self,
        app,
        admin_secret: str,
        base_url: str = "http://127.0.0.1:8769",
        *args,
        **kwargs,
    ):
        super().__init__(app, *args, **kwargs)
        self.base_url = base_url
        self.secret = admin_secret

    async def dispatch(self, request, call_next):
        # 请求前的处理
        # 获取当前用户的信息，根据其状态处理
        request_id = None  # 请求id默认为空
        if hasattr(request.state, "payload"):
            # 上传用户请求信息，并且获取用户最新状态
            user_state_response = requests.post(
                url=self.base_url + "/api/v1/admin/requests",
                headers={"Authorization": f"Bearer {self.secret}"},
                data={"user": request.state.payload},
            )
            if user_state_response.status_code == 200:
                # 用户状态正常，继续处理请求
                user_state = user_state_response.json()
                request.state.user_state = user_state
                if user_state["state"]["value"] == "blacklist":
                    return JSONResponse(
                        status_code=403,
                        content={"message": user_state["state"]["message"]},
                    )
                request_id = user_state["id"]

        response = await call_next(request)

        # 请求后的处理
        # 将响应保存到admin server中

        if request_id:
            requests.post(
                url=self.base_url + "/api/v1/admin/requests",
                headers={"Authorization": f"Bearer {self.secret}"},
                data={
                    "user": request.state.payload,
                    "request_id": request_id,
                    "request": request,
                    "response": response,
                },
            )

        return response



import json
import requests
from typing import TypedDict, Optional


class DataValue(TypedDict):
    value: str


class Data(TypedDict):
    keyword1: Optional[DataValue]
    keyword2: Optional[DataValue]
    keyword3: Optional[DataValue]
    keyword4: Optional[DataValue]
    keyword5: Optional[DataValue]


def api_send_message_by_weixin_base(
    app_id: str,
    app_secret: str,
    henau_openid: str,
    template_number: str,
    jump_link: str,
    data: Data,
    host: str = "https://oauth.henau.edu.cn",
):
    try:
        url = f"{host}/oauth2_server/message?appid={app_id}&secret={app_secret}"

        response = requests.post(
            url,
            data=json.dumps(
                {
                    "henau_openid": henau_openid,
                    "template_number": template_number,
                    "url": jump_link,
                    "data": data,
                }
            ),
        )
        if response.json()["status"] != "error":
            return True

    except Exception as e:
        print(e)
        return False


def api_send_message_by_sms_base(
    app_id: str,
    app_secret: str,
    henau_openid: str,
    template_number: str,
    data: Data,
    host: str = "https://oauth.henau.edu.cn",
):
    try:
        url = f"{host}/oauth2_server/sendsms?appid={app_id}&secret={app_secret}"

        response = requests.post(
            url,
            data=json.dumps(
                {
                    "henau_openid": henau_openid,
                    "sms_template_number": template_number,
                    "data": data,
                }
            ),
        )

        if response.json()["status"] != "error":
            return True

    except Exception as e:
        print(e)
        return False
